@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-""" Repl module defines variants of `instantiate` and `realize` functions, which
+""" Repl module defines variants of `instantiate` and `realize1` functions, which
 are suitable for REPL shells. Repl-friendly wrappers (see `repl_realize`) could
 pause the computation, save the Pylightnix state into a variable and return to
 the REPL's main loop. At this point user could alter the state of the whole
@@ -22,21 +22,21 @@ continue or cancel the realization.
 
 from pylightnix.utils import ( dirrm, timestring, concat )
 
-from pylightnix.types import ( Dict, Closure, Context, Derivation, RRef, DRef,
-    List, Tuple, Optional, Generator, Path, Build, Union, Any, BuildArgs,
-    RealizeArg, Tag, RRefGroup )
+from pylightnix.types import (Dict, Closure, Context, Derivation, RRef, DRef,
+                              List, Tuple, Optional, Generator, Path, Build,
+                              Union, Any, BuildArgs, RealizeArg, SPath,
+                              StorageSettings)
 
-from pylightnix.core import ( realizeSeq, store_realize_group, RealizeSeqGen,
-    groups2rrefs )
-from pylightnix.build import ( mkbuildargs, build_outpaths )
+from pylightnix.core import (realizeSeq, RealizeSeqGen, mkrealization)
 
 class ReplHelper:
   def __init__(self, gen:RealizeSeqGen)->None:
     self.gen:Optional[RealizeSeqGen]=gen
+    self.S:Optional[StorageSettings]=None
     self.dref:Optional[DRef]=None
     self.context:Optional[Context]=None
     self.drv:Optional[Derivation]=None
-    self.rrefgs:Optional[List[RRefGroup]]=None
+    self.result:Optional[Context]=None
     self.rarg:Optional[RealizeArg]=None
 
 ERR_INVALID_RH="Neither global, nor user-defined ReplHelper is valid"
@@ -44,9 +44,9 @@ ERR_INACTIVE_RH="REPL session is not paused or was already unpaused"
 
 PYLIGHTNIX_REPL_HELPER:Optional[ReplHelper]=None
 
-def repl_continueMany(out_groups:Optional[List[Dict[Tag,Path]]]=None,
-                      out_rrefgs:Optional[List[RRefGroup]]=None,
-                      rh:Optional[ReplHelper]=None)->Optional[List[RRef]]:
+def repl_continueAll(out_paths:Optional[List[Path]]=None,
+                      out_rrefs:Optional[List[RRef]]=None,
+                      rh:Optional[ReplHelper]=None)->Optional[Context]:
   global PYLIGHTNIX_REPL_HELPER
   if rh is None:
     rh=PYLIGHTNIX_REPL_HELPER
@@ -55,35 +55,44 @@ def repl_continueMany(out_groups:Optional[List[Dict[Tag,Path]]]=None,
   assert rh.dref is not None, ERR_INACTIVE_RH
   assert rh.context is not None, ERR_INACTIVE_RH
   assert rh.drv is not None, ERR_INACTIVE_RH
+  assert rh.S is not None, ERR_INACTIVE_RH
   try:
-    rrefgs:Optional[List[RRefGroup]]
-    if out_groups is not None:
-      assert out_rrefgs is None
-      rrefgs=[store_realize_group(rh.dref,rh.context,g) for g in out_groups]
-    elif out_rrefgs is not None:
-      assert out_groups is None
-      rrefgs=out_rrefgs
+    rrefs:Optional[List[RRef]]
+    if out_paths is not None:
+      assert out_rrefs is None
+      rrefs=[mkrealization(rh.dref,rh.context,p,rh.S)
+                    for p in out_paths]
+    elif out_rrefs is not None:
+      assert out_paths is None
+      rrefs=out_rrefs
     else:
-      rrefgs=None
-    rh.dref,rh.context,rh.drv,rh.rarg=rh.gen.send((rrefgs,False))
+      rrefs=None
+    rh.S,rh.dref,rh.context,rh.drv,rh.rarg=rh.gen.send((rrefs,False))
   except StopIteration as e:
     rh.gen=None
-    rh.rrefgs=e.value
-  return repl_rrefs(rh)
+    rh.result=e.value
+  return repl_result(rh)
 
-def repl_continue(out_groups:Optional[List[Dict[Tag,Path]]]=None,
-                  out_rrefs:Optional[List[RRefGroup]]=None,
+def repl_continueMany(out_paths:Optional[List[Path]]=None,
+                  out_rrefs:Optional[List[RRef]]=None,
+                  rh:Optional[ReplHelper]=None)->Optional[List[RRef]]:
+  ctx=repl_continueAll(out_paths,out_rrefs,rh)
+  if ctx is None:
+    return None
+  assert len(ctx)==1, f"Expected a single-targeted closure"
+  return ctx[list(ctx.keys())[0]]
+
+def repl_continue(out_paths:Optional[List[Path]]=None,
+                  out_rrefs:Optional[List[RRef]]=None,
                   rh:Optional[ReplHelper]=None)->Optional[RRef]:
-  rrefs=repl_continueMany(out_groups,out_rrefs,rh)
+  rrefs=repl_continueMany(out_paths,out_rrefs,rh)
   if rrefs is None:
     return None
-  assert len(rrefs)==1, f"Acturally {len(rrefs)}"
+  assert len(rrefs)==1, f"Expected a single-result derivation"
   return rrefs[0]
 
-def repl_continueBuild(b:Build, rh:Optional[ReplHelper]=None)->Optional[RRef]:
-  return repl_continue(out_groups=b.outgroups, rh=rh)
 
-def repl_realize(closure:Closure,
+def repl_realize(closure:Union[Closure,Tuple[Any,Closure]],
                  force_interrupt:Union[List[DRef],bool]=True,
                  realize_args:Dict[DRef,RealizeArg]={})->ReplHelper:
   """
@@ -105,27 +114,40 @@ def repl_realize(closure:Closure,
   # holding the result of our hacks.
   ```
   """
+  # FIXME: define a Closure as a datatype and simplify the below check
+  closure_:Closure
+  if isinstance(closure,tuple) and len(closure)==2:
+    closure_=closure[1] # type:ignore
+  else:
+    closure_=closure # type:ignore
   global PYLIGHTNIX_REPL_HELPER
   force_interrupt_:List[DRef]=[]
   if isinstance(force_interrupt,bool):
     if force_interrupt:
-      force_interrupt_=[closure.dref]
+      force_interrupt_=closure_.targets
   elif isinstance(force_interrupt,list):
-    force_interrupt_=list(force_interrupt)
+    force_interrupt_=force_interrupt
   else:
     assert False, "Invalid argument"
-  rh=ReplHelper(realizeSeq(closure,force_interrupt_,realize_args=realize_args))
+  rh=ReplHelper(realizeSeq(closure_,force_interrupt_,realize_args=realize_args))
   PYLIGHTNIX_REPL_HELPER=rh
   assert rh.gen is not None, ERR_INACTIVE_RH
   try:
-    rh.dref,rh.context,rh.drv,rh.rarg=next(rh.gen)
+    rh.S,rh.dref,rh.context,rh.drv,rh.rarg=next(rh.gen)
   except StopIteration as e:
     rh.gen=None
-    rh.rrefgs=e.value
+    rh.result=e.value
   return rh
 
+def repl_result(rh:ReplHelper)->Optional[Context]:
+  return rh.result
+
 def repl_rrefs(rh:ReplHelper)->Optional[List[RRef]]:
-  return groups2rrefs(rh.rrefgs) if rh.rrefgs is not None else None
+  ctx=repl_result(rh)
+  if ctx is None:
+    return None
+  assert len(ctx)==1
+  return ctx[list(ctx.keys())[0]]
 
 def repl_rref(rh:ReplHelper)->Optional[RRef]:
   rrefs=repl_rrefs(rh)
@@ -134,19 +156,6 @@ def repl_rref(rh:ReplHelper)->Optional[RRef]:
   assert len(rrefs)==1
   return rrefs[0]
 
-def repl_buildargs(rh:Optional[ReplHelper]=None, buildtime:bool=True)->BuildArgs:
-  global PYLIGHTNIX_REPL_HELPER
-  if rh is None:
-    rh=PYLIGHTNIX_REPL_HELPER
-  assert rh is not None, ERR_INVALID_RH
-  assert rh.context is not None, ERR_INACTIVE_RH
-  assert rh.dref is not None, ERR_INACTIVE_RH
-  assert rh.rarg is not None, ERR_INACTIVE_RH
-  timeprefix=timestring() if buildtime else None
-  return mkbuildargs(rh.dref, rh.context, timeprefix, {}, rh.rarg)
-
-def repl_build(rh:Optional[ReplHelper]=None, buildtime:bool=True)->Build:
-  return Build(repl_buildargs(rh, buildtime))
 
 def repl_cancel(rh:Optional[ReplHelper]=None)->None:
   global PYLIGHTNIX_REPL_HELPER
@@ -160,9 +169,4 @@ def repl_cancel(rh:Optional[ReplHelper]=None)->None:
   except StopIteration as e:
     rh.gen=None
 
-def repl_cancelBuild(b:Build, rh:Optional[ReplHelper]=None)->None:
-  repl_cancel(rh)
-  for g in b.outgroups:
-    for o in g.values():
-      dirrm(o)
 
